#include <linux/ioctl.h>
#include <linux/cdev.h>

#define PMC_CT_2048_OUT 0x00010000
#define PMC_USE_FOR_SYNC_2048 0x00000200
#define PMC_WORKING_MODE 0x00200000
#define PMC_OLD_FIRMWARE 0x1113

#define PMC_CAS_FIFO_LEN 100
#define PMC_CT_TO_EN 0x08000000
#define PMC_CT_TI_EN 0x04000000
#define PMC_MASL     0x00800000
#define PMC_DMARST   0x02000000
#define PMC_CONS_DEL 0x00008000

enum bar0_regs_t {
    HW_FW_REV = 0x0D
};

struct pmc_e1_cas_data
{
    unsigned char chan_num;
    unsigned char cdata[16];
};

struct pmc_e1_controller
{
	struct pci_dev		*pci_device;	//coresponding pci_dev struct
	struct list_head	dev_node;	//list of devices, may be we don't need it now...
	struct cdev		dev_file;	//descriptop file
	spinlock_t              dev_lock;
	unsigned char           dev_number;
	volatile unsigned int	*bar0;
	volatile unsigned int	*bar1;		//pointers to map device's BARS
	unsigned char           e1_amount;      //number of last e1 if
	unsigned char		initialized:1;
	unsigned char		opened:1;
	unsigned char		registered:1;
	char                    name[30];
	unsigned char           pos_in_write_buf;
	void *                  priv_data;      //hook for dahdi
	unsigned int            test_sig;
	struct device     *pmc_class_dev;
};

#define getp_pmc_struct(pdev) list_entry(pdev, pmc_e1_controller, pci_dev)

#define CTB_TO_E1_TABLE(ct_ch, ct_ts)	( ct_ts | ( ct_ch << 8) )
#define	E1_TO_CTB_TABLE(e1_ch, e1_ts)	( ( e1_ts | 0x00008000) | (e1_ch  << 8) )


#define PMC_BIT_(a)  (1<<a)
#define PMC_GBIT(a)  ((a) & 0x000000ff)
#define PMC_GPAGE(a) (((a)>>16) & 0x000000ff)
#define PMC_GREG(a)  (((a)>>8) & 0x000000ff)
#define PMC_PAGE_REG_BIT(a,b,c) (a<<16|b<<8|1<<c)



/** Hardware positions of configurational bits. */
enum cfg0_hp_t {
  CFG0_CAS_HP = PMC_PAGE_REG_BIT(1,0x14,5),     /** CAS  (1=CSS/0=CAS)   1-14-5*/
  CFG0_XMIT_TR_HP = PMC_PAGE_REG_BIT(1,0x12,2),  /** Transparent on transmit 1=transparent/0=framing on 1-12-2*/
  CFG0_RCV_TR_HP = PMC_PAGE_REG_BIT(1,0x12,3), /** Transparent on receive  1-12-3*/
  CFG0_MASTER_HP = PMC_BIT_(5),  /** Master/Slave = 1/0 on E1*/
  CFG0_RCV_HDB3_HP = PMC_PAGE_REG_BIT(1,0x15,7),   /**Enable HDB encoding on RX 1-15-7*/
  CFG0_XMIT_HDB3_HP = PMC_PAGE_REG_BIT(1,0x15,5),  /**Enable HDB encoding on TX 1-15-5*/
};

/** International/National bits */
enum cfg1_hp_t {
  CFG1_NAT0_HP = PMC_PAGE_REG_BIT(1,0x17,4),   /**Sa4 1-17-4 */
  CFG1_NAT1_HP = PMC_PAGE_REG_BIT(1,0x17,3),   /**Sa5 1-17-3 */
  CFG1_NAT2_HP = PMC_PAGE_REG_BIT(1,0x17,2),   /**Sa6 1-17-2 */
  CFG1_NAT3_HP = PMC_PAGE_REG_BIT(1,0x17,1),   /**Sa7 1-17-1 */
  CFG1_NAT4_HP = PMC_PAGE_REG_BIT(1,0x17,0),   /**Sa8 1-17-0 */
  CFG1_INAT0_HP = PMC_PAGE_REG_BIT(1,0x12,0),  /** 1-12-0*/
  CFG1_INAT1_HP = PMC_PAGE_REG_BIT(1,0x12,1),  /** 1-12-1*/
};

enum cfg2_hp_t {
    CFG2_RAI_AUTO_ON_HP = PMC_PAGE_REG_BIT(1,0x10,4),  /** RAI: Auto or ON: 0=Auto/1=ON  1-10-4*/
    CFG2_RAI_ENA_HP = PMC_PAGE_REG_BIT(1,0x16,5),      /** RAI: 0=normal/1=transmit alarm, ignoring in Auto mode 1-16-5*/
    CFG2_RCAS_AUTO_ON_HP = PMC_PAGE_REG_BIT(1,0x10,3), /** Remote CAS: 0=Auto/0=ON 1-10-3*/
    CFG2_RCAS_ENA_HP = PMC_PAGE_REG_BIT(1,0x13,2),     /** Enable remote CAS (Y bit): 0=normal/1=LOM, ignoring in Auto mode 1-13-2*/
    CFG2_X1_HP = PMC_PAGE_REG_BIT(1,0x13,3),           /** X1 multiframe 1-13-3*/
    CFG2_X2_HP = PMC_PAGE_REG_BIT(1,0x13,1),           /** X2 multiframe 1-13-1*/
    CFG2_X3_HP = PMC_PAGE_REG_BIT(1,0x13,0),           /** X3 multiframe 1-13-0*/
};

/**CRC4 related */
enum cfg3_hp_t {
    CFG3_CRC4_TR_HP = PMC_PAGE_REG_BIT(1,0x10,6), /** CRC4 transparent regeneration on 1-10-6*/
    CFG3_CRC4_DIS_HP = PMC_PAGE_REG_BIT(1,0x10,2),    /** CRC4 1=disable/0=enable  1-10-2*/
    CFG3_CRC4_MANUAL_HP = PMC_PAGE_REG_BIT(1,0x10,5), /**CRC4 Automatic CRC internetworking 0=on/1=off 1-10-5*/
};

/** 4 testing */
enum cfg4_hp_t {
  CFG4_AIS_HP = PMC_PAGE_REG_BIT(1,0x11,4),      /** TxAO 1=disable/0=enable 1-11-4*/
  CFG4_LLOOP_HP = PMC_PAGE_REG_BIT(1,0x15,6),    /** Metallic loopback (shleif)  1=loop/0=normal 1-15-6*/
  CFG4_RLOOP_HP = PMC_PAGE_REG_BIT(1,0x15,2),    /** Remote loopback (echo) 1=loop/0=normal  1-15-2*/
  CFG4_TX_DIS_HP,   /** 1=Disable transmition on the board level, use to emulate LOS, transmition enabled by default*/
  CFG4_PLOOP_HP = PMC_PAGE_REG_BIT(1,0x15,0),    /** Payload loopback 1=loop/0=normal 1-15-0 */
};

enum err_emu0_hp_t {
    ERR_EMU0_LOF_NFAS_HP = PMC_PAGE_REG_BIT(1,0x19,4), /** Emulate single frame violation (1=generate error)  1-19-4*/
    ERR_EMU0_LOF_FAS_HP = PMC_PAGE_REG_BIT(1,0x19,5),  /** Emulate single frame violation (1=generate error) 1-19-5*/
    ERR_EMU0_CRC4_HP = PMC_PAGE_REG_BIT(1,0x19,6),     /** Emulate single CRC4 error (1=generate error)  1-19-6*/
};

/**Misc status bits*/
enum stat0_hp {
	STAT0_LOF_HP = PMC_PAGE_REG_BIT(3,0x10,7),       /** LOF signal 1-loss/0-ok 3-10-7*/
	STAT0_CAS_LOM_HP = PMC_PAGE_REG_BIT(3,0x10,6),   /** CAS LOM signal 1-loss/0-ok 3-10-6*/
	STAT0_CRC4_SYNC_HP = PMC_PAGE_REG_BIT(3,0x10,5), /** CRC4 syncronisation 1-loss/0-ok 3-10-5*/
	STAT0_E_BIT1_HP = PMC_PAGE_REG_BIT(3,0x10,3),     /** bit E state 1=error/0=ok !(3-10-3 &  3-10-4)*/
        STAT0_E_BIT2_HP = PMC_PAGE_REG_BIT(3,0x10,4),
	STAT0_CRC4_1_HP = PMC_PAGE_REG_BIT(3,0x11,6),      /** CRC4 state  1=error/0=ok  (3-11-6 | 3-11-7)*/
        STAT0_CRC4_2_HP = PMC_PAGE_REG_BIT(3,0x11,7),
	STAT0_LOS_HP = PMC_PAGE_REG_BIT(3,0x11,4) ,       /** LOS signal 1-loss/0-ok 3-11-4*/
	STAT0_AIS_HP = PMC_PAGE_REG_BIT(3,0x11,2),       /** AIS state 1=valid AIS or all ones/0=valid AIS is not being received 3-11-2*/
	STAT0_RAIS_HP = PMC_PAGE_REG_BIT(3,0x11,1),      /** RAIS state 1=error/0=normal 3-11-1*/

};

/** International/National bits */
enum stat1_hp {
  STAT1_NAT0_HP = PMC_PAGE_REG_BIT(3,0x18,4),      /**Sa4 3-18-4 */
  STAT1_NAT1_HP = PMC_PAGE_REG_BIT(3,0x18,3),      /**Sa5 3-18-3 */
  STAT1_NAT2_HP = PMC_PAGE_REG_BIT(3,0x18,2),      /**Sa6 3-18-2 */
  STAT1_NAT3_HP = PMC_PAGE_REG_BIT(3,0x18,1),      /**Sa7 3-18-1 */
  STAT1_NAT4_HP = PMC_PAGE_REG_BIT(3,0x18,0),      /**Sa8 3-18-0 */
  STAT1_INAT1_HP = PMC_PAGE_REG_BIT(3,0x18,7),     /**3-18-7*/
  STAT1_INAT0_HP = PMC_PAGE_REG_BIT(3,0x15,7),     /**3-15-7*/
  STAT1_SLIP_HP = PMC_PAGE_REG_BIT(4,0x12,1),      /**Slip occured  1=Slip, auto clearing after reading 4-12-1 */
};

/**Multifreim X Y bits*/
enum stat2_hp {
	STAT2_X1_HP = PMC_PAGE_REG_BIT(3,0x19,3),  /**3-19-3*/
	STAT2_Y_HP = PMC_PAGE_REG_BIT(3,0x19,2),   /**3-19-2*/
	STAT2_X2_HP = PMC_PAGE_REG_BIT(3,0x19,1),  /**3-19-1*/
        STAT2_X3_HP = PMC_PAGE_REG_BIT(3,0x19,0),  /**3-19-0*/
};

/** Overrun bits */
enum ovr_mgmt_hp {
	OVR_HDB3_HP = PMC_PAGE_REG_BIT(4,0x1f,3),  /**4-1f-3*/
	OVR_FAS_HP =  PMC_PAGE_REG_BIT(4,0x1f,7), /**4-1f-7*/
	OVR_CRC4_HP = PMC_PAGE_REG_BIT(4,0x1f,6), /**4-1f-6*/
};

/** Selection of mode & a/b select*/
enum mode1_hp {
	MODE1_AB_HP = PMC_BIT_(22),           /**0=use A signals/1=use B signals*/
	MODE1_MA_SL_SEL_HP = PMC_BIT_(23),    /**0=slave on CT-BUS, 1=Master on CT-BUS*/
	MODE1_NF1_GEN_ENA_HP = PMC_BIT_(18) , /**1=Enable to generate Netref1*/
	MODE1_NF2_GEN_ENA_HP =  PMC_BIT_(17), /**1=Enable to generate Netref2*/
	MODE1_NF1_USE_ENA_HP =  PMC_BIT_(20), /**1=Enable to use  Netref1 for syncronization*/
	MODE1_NF2_USE_ENA_HP =  PMC_BIT_(19), /**1=Enable to use  Netref2 for syncronization*/
	MODE1_FLOCK =  PMC_BIT_(8), 	      /**  */
	MODE1_FREE =  PMC_BIT_(7), 	      /**1=Using Internal synchronization */
	MODE1_HOLD =  PMC_BIT_(6), 	      /**1=Holding last derived synchronization */
};

#define module_printk(level, fmt, args...) printk(level "%s: " fmt, THIS_MODULE->name, ## args)
#ifdef PMC_E1_DEBUG
# define module_dprintk(level, fmt, args...) \
    module_printk(level, fmt, ## args)
# define dprintk(level, fmt, args...) \
    printk(level fmt, ## args)
#else
# define module_dprintk(level, fmt, args...)
# define dprintk(level, fmt, args...)
#endif
